/****************************************************************
ConfigServerMain.C
CS 481 - Team E
Spring 1998
Hummingbird Project - Dr. Frincke
Coded by: Jamie Marconi
Description: This is the main file for the configuration propagation 
             server. The server waits for a connect from the client 
	     and if the client needs to be updated the server sends the
	     new config files to it.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char ConfigServerMain_C[] = "$Id: ConfigServerMain.C,v 1.20 1998/04/26 20:45:48 jamie Exp $";

#include "ConfigServer.h"

#ifdef POSTGRES
#include <Hummer_PG.h>
#endif

#include <sys/types.h>
#include <sys/wait.h>

// Workaround for signal handling. (Since we can't have a pointer to a 
// member function)
ConfigServer *SignalHandleObject;

// Function prototypes
void Signal_Handler(int Sig);

RETSIGTYPE main_sigchld_handler(int sig)
{
  int status;
#ifdef HAVE_WAITPID
  /* Reap all childrens */
  while (waitpid(-1, &status, WNOHANG) > 0)
    ;
#else
  wait(&status);
#endif
  signal(SIGCHLD, main_sigchld_handler);
}



/**********
Function: Main
Purpose:  This is the main function of the configuration propagation server.
          It creates a configuration server object and sets up the signals 
	  to be trapped.
**********/

int main(void)
{
  // Fork server into the background, doesn't die when shell exits.

#undef HAVE_DAEMON
#ifdef HAVE_DAEMON
  // BSD only?  Don't like results very much.
  if (daemon(1, 1) < 0)
    perror("daemon");
#else /* HAVE_DAEMON */


  {
#ifdef TIOCNOTTY
    int fd;
#endif /* TIOCNOTTY */
    
    // Fork, and have the parent exit.  The child becomes the server. 
    if (fork())
      exit(0);
    
    // Redirect stdin, stdout, and stderr to /dev/null. 
    freopen("/dev/null", "r", stdin);
    freopen(HUMMER_LOG, "a", stdout);
    freopen(HUMMER_LOG, "a", stderr);
    
    // Disconnect from the controlling tty. 
#ifdef TIOCNOTTY
    fd = open("/dev/tty", O_RDWR|O_NOCTTY);
    if (fd >= 0)
      {
	(void)ioctl(fd, TIOCNOTTY, NULL);
	close(fd);
      }

    // I don't really know, found it in someone else's code.
#endif /* TIOCNOTTY */
#ifdef HAVE_SETSID
#ifdef ultrix
    setpgrp(0, 0);
#else /* ultrix */
    if (setsid() < 0)
      perror("setsid");
#endif
#endif /* HAVE_SETSID */
  }
#endif /* HAVE_DAEMON */


  // Create the server object
  ConfigServer ConfigurationServer(CONFIG_PORT);
  SignalHandleObject = &ConfigurationServer;

  // Set up the signals we would like to trap.
  // Restart and stop we handle
  signal(SIGCHLD, main_sigchld_handler);
  signal(SIGHUP, Signal_Handler); 
  signal(SIGINT, Signal_Handler);

  // Start the server
  ConfigurationServer.StartServer();

  return 0;
}


/**********
Function: Signal_Handler
Purpose:  This function handles signals sent to the config server.
**********/

void Signal_Handler(int Sig)
{
  switch(Sig){
    // We don't have a restart task anymore.
  case 1:
    //SignalHandleObject->ReConfigSubs();
    signal(SIGHUP, Signal_Handler); 
    break;
    // Set values to kill the server
  case 2:
    // finally set kill value for server loop
    SignalHandleObject->KillServer();
    break;
  }
}

