/****************************************************************
ConvertIP.C
CS 481 - Team A
Spring 1997
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen
Description: This files contains the ConvertIP function that makes an 
             IP into a string so we can compare them with strcmp. 
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char ConvertIP_C[] = "$Id: ConvertIP.C,v 1.6 1998/03/06 23:00:08 marc9442 Exp $";

#include "ConvertIP.h"
#include "Print_Error.h"
#include <strings.h>
/**********
Function: ConvertIP
Purpose:  Convert an IP address in the form XXX.XXX.XXX.XXX to the form
          XXXXXXXXXXXX so it can be compared using strcmp.
**********/

void ConvertIP(const char *Addr, char *NewAddr)
{
  char *TempAddr;
  char *TempStr;
  int i;
#ifdef DEBUG_CONVERTIP
  Print_Error("ConvertIP", "Passed Address: %s", Addr);
#endif
  // New some temp memory for an address
  TempAddr = new char[IP_ADDRESS_SIZE];
  
  // Look up the address with DNS to get numeric IP addr
  struct hostent *host;
  if ((host = gethostbyname(Addr)) == NULL) {
    // Address was unknown just use plain address
    strcpy(TempAddr, Addr);
  }
  else {  
    sockaddr_in sin;
    bzero((char *) &sin, sizeof (struct sockaddr_in));
    memcpy((char *) &sin.sin_addr, host->h_addr, sizeof(sin.sin_addr));
    // Copy over number form
    strncpy(TempAddr, inet_ntoa(sin.sin_addr), IP_ADDRESS_SIZE);
  }

#ifdef DEBUG_CONVERTIP
  Print_Error("ConvertIP", "Looked up Address: %s", TempAddr);
#endif

  // Set NewAddr to nothing
  strcpy(NewAddr, "");
  // Tokenize the TempAddr
  TempStr = strtok(TempAddr,".");

  // For each of the four octets of the IP address
  for(i=0;i<4;i++){
    // Pad out the address octet with zeros
    if(strlen(TempStr) == 3) strcat(NewAddr, TempStr);
    else if(strlen(TempStr) == 2){
      strcat(NewAddr, "0");
      strcat(NewAddr, TempStr);
    }
    else{
      strcat(NewAddr, "00");
      strcat(NewAddr, TempStr);
    }
    // Get next octet
    TempStr = strtok(NULL,".");
  }

#ifdef DEBUG_CONVERTIP
  Print_Error("ConvertIP", "Converted Address: %s", NewAddr);
#endif
  
  // Free memory
  delete[] TempAddr;
  
}
