/****************************************************************
Hummer_PG.C
CS 481 - Team E
Spring 1998
Hummingbird Project - Dr. Frincke
Coded by: Jamie Marconi
Description: This files contains the basic functions that makes a
             Database connection and Queries
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char Hummer_PG_C[] = "$Id: Hummer_PG.C,v 1.7 1998/03/06 23:00:09 marc9442 Exp $";

#include "Hummer_PG.h"

/**********
Function: OpenDB
Purpose:  Establish a connection to the PostgreSQL database backend
          returns a PgDatabase pointer, or NULL if error.
**********/

PgDatabase *OpenDB(void)
{
  // Get the local hostname
  char *HOSTNAME;
  HOSTNAME = new char[MAXHOSTNAMELEN];
  gethostname(HOSTNAME, MAXHOSTNAMELEN);

  // Construct the database name to attach to.  The form is hummer_hostname
  char *dbname;
  dbname = new char[strlen(DBNAMEPRE)+strlen(HOSTNAME)+1];
  strcpy(dbname, DBNAMEPRE);
  strcat(dbname, HOSTNAME);

  // We don't need the hostname anymore, now that we have dbname
  delete [] HOSTNAME;

  // Connect to DB with dbname on local host.
  PgDatabase *db;
  db = new PgDatabase(dbname);
  
  // Check if the connection was successful.
  if ( db->ConnectionBad() ) {
    // if there was a problem report it and die.
    Print_Error("OpenDB", 
		"Connection to database %s was unsuccessful.", dbname);
    Print_Error("OpenDB", 
		"Error message returned: %s", db->ErrorMessage());

    delete db;
    delete [] dbname;
    
    return NULL;
  }
  
  else {
#ifdef DEBUG_QUERY
    Print_Error("OpenDB", "Connection to database was successful.");
#endif /* DEBUG */

    // we don't need dbname anymore
    delete [] dbname;

    // we got the successful connection now return it.
    return db;
  }

}

/**********
Function: QueryDB
Purpose:  Execute the query paramete as an SQL statement on the PassedDB 
          database connection.  Will return -2 on error, -1 on successful
	  command, and > 0 for the number of tuples returned.
**********/

int QueryDB(PgDatabase **PassedDB, char *query)
{
  ExecStatusType status;

  // pass of passed DB connection to db
  PgDatabase *db = *PassedDB;

  if (!db) {
    return -2;
  }

#ifdef DEBUG_QUERY
  Print_Error("QueryDB", query);
#endif
  // Execute the query
  status = db->Exec(query);

#ifdef DEBUG_QUERY
  Print_Error("QueryDB", "%d returned on query", status);
#endif
  
  // what was the return status
  if (status == PGRES_TUPLES_OK ) {

#ifdef DEBUG_QUERY
    Print_Error("QueryDB", "TUPLES OK: %d", db->Tuples());
#endif
    // return the number of tuples
    return db->Tuples();
  }
  
  else if (status == PGRES_COMMAND_OK) {
#ifdef DEBUG_QUERY
    Print_Error("QueryDB","COMMAND OK");
#endif
    
    // Command Ok means query was successful, but not a tuple returning query
    return -1;
  }

  // Anything else is an error
  else {
    Print_Error("QueryDB", "Query Error: %s", db->ErrorMessage());

    return -2;
  }
}
