/****************************************************************
MessageHandler.C
CS 481 - Team A
Spring 1997
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen and Jamie Marconi
Description: The message handler is the bulk of the messaging server.
             It matches a message to a number of filters and then
	     uses these filters to process the message. 
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char MessageHandler_C[] = "$Id: MessageHandler.C,v 1.30 1998/03/06 23:00:12 marc9442 Exp $";

#include "MessageHandler.h"

extern char *HOSTNAME;


/**********
Function: MessageHandler::MessageHandler
Purpose:  Basic constructor
**********/

MessageHandler::MessageHandler()
{
  Kills = NULL;
}


/**********
Function: MessageHandler::~MessageHandler
Purpose:  Basic destructor
**********/

MessageHandler::~MessageHandler()
{
  HostList *Temp;

  // Delete Kill File memory
  HostList *DelPtr;
  
  Temp = Kills;
  while (Temp != NULL) {
    DelPtr = Temp;
    Temp = Temp->Next;
    delete DelPtr;
  }
  Kills = NULL;
}

/**********
Function: MessageHandler::InKills
Purpose:  This function takes an IP address as a parameter and returns true
          if associated with that address there is an entry in the kill file.
**********/

int MessageHandler::InKills(char *HostIP)
{
  HostList *TempPointer;
  int In;
  char *Host;

  // Allocate memory
  Host = new char[IP_ADDRESS_SIZE];

  TempPointer = Kills;
  
  ConvertIP(HostIP, Host);
  // Its not in the kill file by default
  In = 0;

  // For Every host in the kill file
  while(TempPointer != NULL){
    // Is the entry an unique host name
    if(!strcmp(TempPointer->EndHostIP,"0")){
      if(!strcmp(TempPointer->StartHostIP, Host)){ 
	In = 1;       
	break;
      }
    }
    // if the entry in a range of hosts
    else{
      if(strcmp(TempPointer->StartHostIP, Host) <= 0 && 
	 strcmp(TempPointer->EndHostIP, Host) >= 0) {
	In = 1;
	break;
      }
      
    }
    TempPointer = TempPointer->Next;
  }

  // Free memory
  delete[] Host;
  
  return In;
}

/**********
Function: MessageHandler::HandleMessage
Purpose:  This function is where all the message handling takes place. It
          uses the message filters that are mathced with the message to 
	  process the message. This function writes to the file: hummer.log
**********/

int MessageHandler::HandleMessage(Message *NewMessage, char *HostIP)
{
  char *OutMessage;
  char *ErrString;
  char *Path;
  int MessageTrust;
  int MessageCoop;
  int MessageInteg;
  int Flag;
  int written = 0, alerted = 0;
  sock_cli *ForwardSocket;
  MessageFilter *FilterList;

  HostList *Hosts;
  FILE *LogFile;

  //  Check and see if this host is in the kill file, if so we are done
  if (InKills(HostIP)) {
#ifdef DEBUG_KILLS
    Print_Error("HandleMessage", "%s is in Kill File", HostIP);
#endif
    return 0;
  }

  // Assign Levels to the message.
  MessageTrust = Levels.GetTrust(HostIP);
  MessageCoop = Levels.GetCoop(HostIP);
  MessageInteg = Levels.GetInteg(HostIP);

  // Match it against the regex's we have. And get threaded list.
  FilterList = Filters.MatchExp(NewMessage->Description);

  // For each returned filter in the thread do
  if(FilterList != NULL){
    while(FilterList != NULL){
      // Check the message against the filters.
      
      // Check the trust level.
      if( FilterList->TrustLevel[0] !='~' &&
	  MessageTrust > atoi(FilterList->TrustLevel)){
	FilterList = FilterList->NewNext;
	continue;
      }

      // Check the coop level.
      if( FilterList->CoopLevel[0] !='~' &&
	  MessageCoop > atoi(FilterList->CoopLevel)){
	FilterList = FilterList->NewNext;
	continue;
      }

      // Check the integ level.
      if( FilterList->IntegLevel[0] !='~' &&
	  MessageInteg > atoi(FilterList->IntegLevel)){
	FilterList = FilterList->NewNext;
	continue;
      }
      
      // Check the date.
      if( FilterList->StartDate[0] != '~' && 
	  (strcmp(FilterList->StartDate, NewMessage->Date) > 0  ||
	  strcmp(FilterList->EndDate, NewMessage->Date) < 0)){
	FilterList = FilterList->NewNext;
	continue;
      }

      // Check the time.
      if( FilterList->StartTime[0] != '~' &&
	  (strcmp(FilterList->StartTime, NewMessage->Time) > 0 ||
	  strcmp(FilterList->EndTime, NewMessage->Time) < 0)){
	FilterList = FilterList->NewNext;
	continue;
      }

      // Check the from hosts list.
      if( FilterList->FromHostList != NULL ){

	// Convert the message host address over to the compair format
	Hosts = FilterList->FromHostList;
	Flag = 0;
	char MesHostIP[IP_ADDRESS_SIZE];
	ConvertIP(NewMessage->HostIP,MesHostIP);

	// For every host in the from list
	while( Hosts != NULL ){ 
	  // Check to see if it matches uniquely
	  if(!strcmp(Hosts->EndHostIP,"0")){
	    if(!strcmp(MesHostIP,Hosts->StartHostIP)) 
	      Flag = 1;
	  }
	  // Check to see if it matches in the range
	  else{
	    if( strcmp(MesHostIP, Hosts->StartHostIP) >= 0 && 
		strcmp(MesHostIP, Hosts->EndHostIP) <= 0 ) 
	      Flag = 1;
	  }
	  Hosts = Hosts->Next;
	}

	// if we set the flag we are ok go
	if( !Flag ){
	  FilterList = FilterList->NewNext;
	  continue;

	}
      }

      // We made it through the filters, carry out the actions specified.

      // Log the message.
      if((FilterList->Log == '1') && (!written)) {
#ifdef POSTGRES	
	char *entry;
 	entry = new char[8192];
	// Convert the date/time over to something the DB can handle
	char *date = makedate(NewMessage->Date);
	char *time = maketime(NewMessage->Time);

	// Open the connection
	PgDatabase *logdb = OpenDB();
	if (!logdb) {
	  //exit(1);
	}
	// If sucessful form our insert query
	else {
	  sprintf(entry, "INSERT INTO log VALUES (%d, %d, %d, '%s', '%s', '%s', '%s', '%s', '%s');",
		  MessageTrust,
		  MessageCoop,
		  MessageInteg,
		  date,
		  time,
		  NewMessage->Description,
		  NewMessage->HostIP,
		  NewMessage->Filename,
		  NewMessage->Tool);	
	  
	  // Run the query
	  if (QueryDB(&logdb, entry)==-1) 
	    written = 1;

	  delete logdb;
	  delete [] entry;
	  delete [] date;
	  delete [] time;
	}
#else
	///////////////////////////////////////////////////////
        // Old File way of doning things to be removed
	//
	Path = new char[NORMAL_STRING_SIZE];
	sprintf(Path, "./%s/%s", HOSTNAME, LOG_FILE);
	LogFile = fopen(Path,"a");
	delete[] Path;
	if(LogFile){
	  fprintf(LogFile, "%d|%s|%s|%s|%s|%s|%s\n", 
		  MessageTrust,
		  NewMessage->Date,
		  NewMessage->Time,
		  NewMessage->Description,
		  NewMessage->HostIP,
		  NewMessage->Filename,
		  NewMessage->Tool);
	
	  fclose(LogFile);
	  written = 1;

	}
	else{
	  // Can open log file for writing. Send an error.
	  //ReportError("hummer.log cannot be written.");
	}
	//
	//
	///////////////////////////////
#endif
      }

      // Alert the console's alert interface.
      if((FilterList->Alert == '1') && (alerted != 1)){

	// Open a socket connection to the interface on the localhost
	ForwardSocket = new sock_cli();
	ForwardSocket->create();

	if(ForwardSocket->connect(HOSTNAME, ALERT_INTERFACE_PORT, ALERT_INTERFACE_SERVICE)){
	  
	  // Authenticate the interface
	  if (ForwardSocket->send_auth()) {
	    OutMessage = new char[BUFFER_SIZE];
	    
	    // Create our message to the interface
	    sprintf(OutMessage, "%d|%d|%d|%s|%s|%s|%s|%s|%s",
		    MessageTrust,
		    MessageCoop,
		    MessageInteg,
		    NewMessage->Date,
		    NewMessage->Time,
		    NewMessage->Description,
		    NewMessage->HostIP,
		    NewMessage->Filename,
		    NewMessage->Tool);
	    
	    // Give it the message
	    ForwardSocket->write(OutMessage);
	    delete[] OutMessage;
	  }
	}
	else{
	  // Can't connect to the interface. Send an error.
	  //ReportError("Hummer cannot connect to alert interface.");
	}
	
	// Close out the socket connection
	ForwardSocket->close();
	delete ForwardSocket;		
	alerted = 1;
      }
      
      // Forward the message to other hosts.
      if(FilterList->ForwardHostList != NULL) {
	Hosts = FilterList->ForwardHostList;

	// For each host in the forward list do
	while(Hosts != NULL) {
	  
	  // Make a connection to the Hummer on the remote system
	  ForwardSocket = new sock_cli();
	  ForwardSocket->create();
	  
	  if (ForwardSocket->connect(Hosts->StartHostIP, MESSAGE_PORT, MESSAGE_SERVICE)) {
	    OutMessage = new char[BUFFER_SIZE];
	    
	    // Set up the outgoing message.
	    sprintf(OutMessage, "%d|%s|%s|%s|%s|%s|%s", 
		    NewMessage->MessageAge,
		    NewMessage->Date,
		    NewMessage->Time,
		    NewMessage->Description,
		    NewMessage->HostIP,
		    NewMessage->Filename,
		    NewMessage->Tool);

	    // Authenticate ourself to the remote system
	    if (!ForwardSocket->send_auth()) {
	      Print_Error("HandleMessage", "Can't forward message to host: %s.",NewMessage->HostIP);		 	    
	    }
	    else
	      // Write out message to remote host 
	      ForwardSocket->write(OutMessage);
	    delete[] OutMessage;
	    
	  }
	  else {
	    // Can't connect to host. Send an error.
	    Print_Error("HandleMessage", "Can't forward message to host: %s.",NewMessage->HostIP);	
	  }

	  // Close out connection to remote host
	  ForwardSocket->close();
	  delete ForwardSocket;
	  Hosts = Hosts->Next;
	}
      }
      
      // Forward the message to peer groups
      if(FilterList->ForwardPeerList != NULL) {
	PeerList *Mods = FilterList->ForwardPeerList;

	// For each moderator of a peer group
	while(Mods != NULL) {

	  // Create a connection to the the moderator
	  ForwardSocket = new sock_cli();
	  ForwardSocket->create();
	  
	  if (ForwardSocket->connect(Mods->ModHostIP, PEER_PORT, PEER_SERVICE)) {
	    
	    // Authenicate ourself to the moderator
	    if (!ForwardSocket->send_auth()) {
	      Print_Error("HandleMessage",
			  "Can't forward message to peer group %s moderated by %s",Mods->GroupName,Mods->ModHostIP);
	    }
	    else {
	      char InMessage[NORMAL_STRING_SIZE];
	      char ServDir[5]; 

	      // Tell the PeerServer that we want to send a message
	      ForwardSocket->write("m");
	      // Tell the PeerServer which group we want to send the message to
	      ForwardSocket->write(Mods->GroupName);
	      // The PeerServer tells us if we are allowed to send a message, disconnects if not
              ForwardSocket->read(ServDir,5);
	      ForwardSocket->read(InMessage,NORMAL_STRING_SIZE);
	      Print_Error("HandleMessage","Connected To Server: %s, %s",ServDir,InMessage);

	      // If we are connected, send the server our message to be forwarded to the peer group
	      ForwardSocket->write(NewMessage->Description);
	      ForwardSocket->write(NewMessage->Date);
	      ForwardSocket->write(NewMessage->Time);
	      ForwardSocket->write(NewMessage->Filename);
	      ForwardSocket->write(NewMessage->Tool);
	      ForwardSocket->write(NewMessage->HostIP);
	      sprintf(ServDir,"%d",NewMessage->MessageAge);
	      ForwardSocket->write(ServDir); 
	      ForwardSocket->close();	     
	      Print_Error("HandleMessage","Message Sent to peer group %s moderated by %s",Mods->GroupName,Mods->ModHostIP);
	    }
	  }
	  else {
	    // Can't connect to host. Send an error.
	    Print_Error("HandleMessage", "Can't forward message to peer group %s moderated by %s",Mods->GroupName,Mods->ModHostIP);
	  }
	  // Close out the socket to the moderator
	  ForwardSocket->close();
	  delete ForwardSocket;
	  Mods = Mods->Next;
	} /* End While Hosts */
      } /* End If ForwardPeerList */
      
      FilterList = FilterList->NewNext;
    } /* End While FilterLists */
    
  }  /* End If FilterList */
  return 1; 
}


/**********
Function: MessageHandler::ReportError
Purpose:  This function reports errors about the messaging server by feeding
          messages back into the hummer using the HandleMessage function.
	  !!!! This method is not use any more, it has been replaces with 
	  !!!! Print_Error.
**********/

void MessageHandler::ReportError(char *ErrMessage)
{
  Message *TempMessage;
  char *TempString;
  tm *TimeStruct;
  time_t *TempTime;
  TempTime = new time_t;

  TempMessage = new Message;
  TempString = new char[255];

  time(TempTime);
  TimeStruct = localtime(TempTime);

  TempMessage->MessageAge = 1;
  sprintf(TempString, "%.4d%.2d%.2d", TimeStruct->tm_year+1900, TimeStruct->tm_mon+1, TimeStruct->tm_mday); 
  strcpy(TempMessage->Date, TempString);
  sprintf(TempString, "%.2d%.2d%.2d", TimeStruct->tm_hour, TimeStruct->tm_min, TimeStruct->tm_sec); 
  strcpy(TempMessage->Time, TempString);
  sprintf(TempString, "Internal Hummer Message: %s", ErrMessage);
  strcpy(TempMessage->Description, TempString);
  strcpy(TempMessage->HostIP, ""); // Make this the host ip later.
  strcpy(TempMessage->Filename, "");

  // Send the message to the handler.
  HandleMessage(TempMessage, "0.0.0.0");

  delete TempMessage;
  delete[] TempString; 

}

