/****************************************************************
ServerMain.C
CS 481 - Team A
Spring 1997
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen
Description: This file contains the main function for the messaging server
             of the hummer. It also contains the signal trapping functions
	     that the server uses.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char ServerMain_C[] = "$Id: ServerMain.C,v 1.24 1998/04/26 20:45:48 jamie Exp $";

#include "Hummer.h" // Global defines for hummer.
#include "Server.h" // Defines for the Server object

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <sys/wait.h>

#ifdef POSTGRES
#include <Hummer_PG.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif /* HAVE_SYS_IOCTL_H */

#ifdef HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif /* HAVE_SYS_IOCTL_H */

#ifdef HAVE_TERMIOS_H
#include <termios.h>
#define USING_TERMIOS
#endif /* HAVE_TERMIOS_H */

// Set up the hostname for pathnames.
char *HOSTNAME;

// This is used for signal handling.
// This is a work around because you can't have a pointer to a 
// member function of an object.
Server *SignalHandleObject;

// Function prototypes.
void Signal_Handler(int Sig);

RETSIGTYPE main_sigchld_handler(int sig)
{
  int status;
#ifdef HAVE_WAITPID
  /* Reap all childrens */
  while (waitpid(-1, &status, WNOHANG) > 0)
    ;
#else
  wait(&status);
#endif
  signal(SIGCHLD, main_sigchld_handler);
}

/**********
Function: Main
Purpose:  This is the main function of the messaging server. It creates the
          Server object, sets up the signal handlers, writes the process pid
	  to file and starts the server running.
**********/

int main(void)
{
  // Set hostname.
  HOSTNAME = new char[MAXHOSTNAMELEN];

  gethostname(HOSTNAME, MAXHOSTNAMELEN);
 #undef HAVE_DAEMON 
#ifdef HAVE_DAEMON
  if (daemon(1, 1) < 0)
    perror("daemon");
#else /* HAVE_DAEMON */
  {
#ifdef TIOCNOTTY
    int fd;
#endif /* TIOCNOTTY */

#ifndef PURIFY    
    /* Fork, and have the parent exit.  The child becomes the server. */
    if (fork())
      exit(0);
    
    /* Redirect stdin, stdout, and stderr to /dev/null. */
    freopen("/dev/null", "r", stdin);
    freopen(HUMMER_LOG, "a", stdout);
    freopen(HUMMER_LOG, "a", stderr);
    
    /* Disconnect from the controlling tty. */
#ifdef TIOCNOTTY
    fd = open("/dev/tty", O_RDWR|O_NOCTTY);
    if (fd >= 0)
      {
	(void)ioctl(fd, TIOCNOTTY, NULL);
	close(fd);
      }
#endif /* TIOCNOTTY */
#ifdef HAVE_SETSID
#ifdef ultrix
    setpgrp(0, 0);
#else /* ultrix */
    if (setsid() < 0)
      perror("setsid");
#endif
#endif /* HAVE_SETSID */
#endif /* PURIFY */
  }
#endif /* HAVE_DAEMON */

  // Setup server.
  Server MessageServer(MESSAGE_PORT);  
  SignalHandleObject = &MessageServer;

  // Set up the signals we would like to trap.
  signal(SIGCHLD, main_sigchld_handler);
  signal(SIGHUP, Signal_Handler); 
  signal(SIGINT, Signal_Handler);
  
  // Start the server.
  MessageServer.StartServer();
  return 0;
}

/**********
Function: Signal_Handler
Purpose:  This function handles signals when they are received by the 
          server process.
**********/

void Signal_Handler(int Sig)
{
  switch(Sig){
  case 1:
    
    // Reload the configuration files. They have been updated.
    SignalHandleObject->SetReloadHandler();
    signal(SIGHUP, Signal_Handler); 
    break;
  case 2:
    
    // Exit the server.
    SignalHandleObject->KillServer();
    break;
  }
}
