// ***************************************************************************
// AddWindow Class
// 
//
// Class Description:  
// ------------------
// Purpose: This class is a subwindow for the Add tool feature found in the 
// ToolInterface class.  When a user Adds a tool and presses continue this class
// becomes instantiated and will popup a dialog box for the user.  This class 
// instantiates and lays out the GUI components for this interface/dialog.  In
// addition this class implements its own event handling and when instantiated 
// this class takes control of the focus of the ToolInterface window.
//
// System Relationship:  Intercommunication with the whole system is minimal.  The
// only communication is with the ToolInterface class.  When this interface is 
// destroyed control is returned to ToolInterface.
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.0.
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  ToolInterface.class.
//
// Post Conditions: False.
//
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class is a subsequent window for the Add tool feature.  It is a dialog box 
// that requests and receives focus when it is instantiated.  It prompts the user 
// for the following:  Tool Class, Directory, and Default Execution.  Each item is
// a TextField for the user to enter the appropriate information into.  The Tool
// Class TextField defaults to the system Tool Class default as does the Directory. 
// These two items can be changed under Options in the menu bar on the main interface.
// Finally, the Default Execution TextField defaults to the name of the audit tool,
// but should contain the execution command for the system to execute.
//
//
// Constructor
// -----------
//      public AddWindow (Frame parent, String title)
//
// Event Handling 
// --------------
//      public boolean action(Event evt, Object arg)
//      public boolean gotFocus(Event event, Object arg)
//
// *****************************************************************************

import java.awt.*;
import java.lang.*;
import java.awt.*;
import java.awt.event.*;

public class AddWindow extends Dialog implements ActionListener, FocusListener {

  Panel      addPanel           = new Panel();                     // Sub-container for the components
  TextField  toolClassTextField = new TextField(Atom.toolClass);   // TextField for Tool Class
  TextField  defaultTextField   = new TextField(30);               // TextField for the Defualt Execution
  TextField  directoryTextField = new TextField(Atom.directory);   // TextField for the Directory

  Button  addSubmitButton;                   // Submit Button
  Button  cancelButton;                      // Cancel Button
 
  public static String toolClassInfo = "";   // Local variables 
  public static String directoryInfo = "";
  public static String defaultInfo   = "";

  Constrain con = new Constrain();

  // *****************************************************************************
  // public AddWindow (Frame parent, String title)
  //
  // Constructor Description:  The constructor for the AddWindow class.  Initializes
  // new variables and components, and lays out the components on the AddWindow dialog
  // interface.
  //
  // Constructor Specification:  The constructor lays out the components for 
  // each area on the AddWindow interface.  It primarily uses the GridBagLayout Manager, 
  // and the FlowLayout Manager for the "Sumbit" or "Cancel" buttons.  The following is 
  // a listing of the interface layout and its corresponding components:
  // 
  //   Position  Description             Components
  //   --------  -----------             ----------
  //    * Frame  AddWindow Interface:    GridBagLayout Manager
  //    * Center Tool Class TextField    toolClassTextField
  //             Default TextField       defaultTextField
  //             Directory TextField     directoryTextField
  //    * South  Add Submit Button       addSubmitButton
  //             Cancel Button           cancelButton
  //           
  // The constructor will be accessed each time the Add audit tool feature in the 
  // ToolInterface is activated.
  // 
  // Pre-Conditions:  Add submit button in the Add feature of the ToolInteface is
  // activated.
  //
  // Post-Conditions: Information is stored in the Local variables.
  //
  // *****************************************************************************

  public AddWindow (Frame parent, String title) {
    super(parent, title, true);
    Panel p = new Panel();

    this.setLayout(new GridBagLayout());
    addPanel.setLayout(new GridBagLayout());
    p.setLayout(new FlowLayout());

    // *** Initialize Buttons ****
    addSubmitButton = new Button("Submit");
    addSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    addSubmitButton.setForeground(Color.blue);
    
    cancelButton = new Button("Cancel");
    cancelButton.setFont(new Font("Dialog", Font.BOLD, 12));
    cancelButton.setForeground(Color.blue);

    String name = ToolInterface.auditToolName;
    defaultTextField.setText(name);

    // LISTENERS
    addSubmitButton.addActionListener(this);
    cancelButton.addActionListener(this);

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.               
    con.constrain(addPanel, new Label("Tool Class"), 0,0,1,1, 0,20,0,0);
    con.constrain(addPanel, toolClassTextField, 0,1,1,1, 0,20,0,0); 
    con.constrain(addPanel, new Label("Directory"), 0,2,1,1, 10,20,0,0); 
    con.constrain(addPanel, directoryTextField, 0,3,1,1, 0,20,0,0);    
    con.constrain(addPanel, new Label("Default Execution"), 0,4,1,1, 10,20,0,0);
    con.constrain(addPanel, defaultTextField, 0,5,1,1, 0,20,0,0);

    // Place the submit or cancel buttons
    p.add(addSubmitButton);
    p.add(cancelButton);

    con.constrain(this, addPanel, 0,0,1,1, GridBagConstraints.NONE,
                          GridBagConstraints.NORTH, 1.0, 1.0, 15, 5, 5, 5);
    
    con.constrain(this, p, 0,1,1,1, GridBagConstraints.NONE,
			    GridBagConstraints.NORTH, 1.0, 1.0, 15, 5, 5, 5);
    this.pack();

  } // constructor


  // ***************************************************************************** 
  //  public void actionPerformed(ActionEvent evt)
  //
  // Method Description: This method handles all of the events of type ActionEvent
  // generated by the components in the AddWindow interface.
  //
  // Method Specification:  If an ActionEvent occurs this method will handle it.
  // Events are generated by the user activating a component.  Such as, clicking a
  // button, or selecting a list item.  The following lists the events and reactions
  // for each ActionEvent:
  //
  //        Label       Component/Event      Reaction
  //        -----       ---------------      --------
  //      * Add Submit  addSubmitButton      All the TextField information is collected and
  //                                         stored in the local static variables.    
  //      * Cancel      cancelButton         Closes/destroys the dialog interface and returns
  //                                         control to the ToolInterface window
  //
  // This method will be accessed frequently, especially in the beginning when alot of 
  // tools are being added.
  //
  // Pre-Condition:  Component objects have been instantiated.
  //
  // Post-Condition: Information has been collected from the TextFields and stored in the 
  // local variables.
  //
  // *****************************************************************************

  public void actionPerformed(ActionEvent evt) {
    Object source = evt.getSource();

    // -----------------
    // Add Submit Button
    // -----------------

    // Collect information from the TextFields.
    // Collection Format: <Tool Class> <Directory> <Default>
    if (source == addSubmitButton) {
      toolClassInfo = toolClassTextField.getText();
      directoryInfo = directoryTextField.getText();
      defaultInfo   = defaultTextField.getText();

      // Validity check
      if (toolClassInfo.equals("")) {
	Atom.systemStatusTextArea.append("Using default Tool Class: " + Atom.toolClass + ". \n");
      }
	  
      if (directoryInfo.equals("")) {
	Atom.systemStatusTextArea.append("Using default Directory: " + Atom.directory + ". \n");
      }
	  
      if (defaultInfo.equals("")) {
	Atom.systemStatusTextArea.append("Please enter a Default execution command. \n");
	return;
      }
      
      // Set proceed Flag to true or 1;
      // This means the user pressed the add submit button, so continue with the addition of the
      // audit tool to the audit_tools.dat file.
      ToolInterface.proceed = 1;
	  
      this.setVisible(false); 
      this.dispose();
     
    } // addSubmitButton


    // --------------------
    // Cancel Button
    //---------------------

    if (source == cancelButton) {
      ToolInterface.proceed = 0;              // The user pressed the cancel button so do not
      this.setVisible(false);                 // proceed with the addition of the audit 
      this.dispose();                         // tool information
    }
    
  } // action


  // Focuse methods
  public void focusGained(FocusEvent evt) { }

  public void focusLost(FocusEvent evt) { }


} // AddPopup













