/*
 * lib/krb5/krb/copy_ds.c
 *
 *
 * krb5_copy_ds()
 */

/*
  Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "k5-int.h"
#include "auth_con.h"

krb5_error_code
krb5_copy_ds(ctx, nctx, dtype)
     const krb5_pointer ctx;
     krb5_pointer *nctx;
     krb5_magic dtype;
{
  krb5_error_code kret;
  krb5_context ser_ctx;
  krb5_octet *outrep;
  size_t outlen;

  if (!ctx) {
    *nctx = 0;
    return 0;
  }

  /* Initialize context and initialize all Kerberos serializers */
  krb5_init_context(&ser_ctx);
  krb5_ser_context_init(ser_ctx);
  krb5_ser_db_context_init(ser_ctx);
  krb5_ser_auth_context_init(ser_ctx);
  krb5_ser_ccache_init(ser_ctx);
  krb5_ser_rcache_init(ser_ctx);
  krb5_ser_keytab_init(ser_ctx);
  
  /* Externalize the data */
  kret = krb5_externalize_data(ser_ctx, ctx, &outrep, &outlen);
  if (kret) {
#ifdef DEBUG
    com_err("krb5_copy_data", kret, "while externalizing data");
#endif
    return kret;
  }

  /* Now attempt to re-constitute it */
  kret = krb5_internalize_opaque(ser_ctx,
				 dtype,
				 (krb5_pointer *) nctx,
				 &outrep,
				 &outlen);

  if (outlen) {
#ifdef DEBUG
    fprintf(stderr, "krb5_copy_data: %d bytes left over after internalize\n", outlen);
#endif
    krb5_xfree(outrep);
  }
  
  if (kret) {
#ifdef DEBUG
    com_err("krb5_copy_data", kret, "while internalizing data");
#endif
    return kret;
  }
   
  krb5_free_context(ser_ctx);

  return 0;
}
