/*************************************************************
krb5_sock.C
HummingBird Project - Deborah A. Frincke, PhD.
Summer 1997
Coded by: Jamie Marconi
Description: This file is the implementation fo the base kerberized socket
class this is inherited by krb5_sock_serv and krb5_sock_cli.
***************************************************************/
/*
  Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <config.h>

#ifdef KERBEROS

#include "krb5_sock.h"

/********
Function: krb5_sock::krb5_sock
Purpose:  Basic constructor
********/

krb5_sock::krb5_sock()
{
  sock = -1;
  context = NULL;
  auth_context = NULL;
  keytab = NULL;
  sin_len = sizeof (struct sockaddr_in);
  rmt_sockaddr = new (struct sockaddr_in);
  loc_sockaddr = new (struct sockaddr_in);
}

/*******
Function: krb5_sock::~krb5_sock
Purpose:  Basic destructor
*******/

krb5_sock::~krb5_sock()
{
  if (context)
    krb5_free_context(context);
  context = NULL;
  sock = -1;
}


/*******
Function: krb5_sock::krb5_create()
Purpose:  Call the kerberos library initializaton functions. Also creates a 
socket file descriptor.
*******/

int krb5_sock::krb5_create()
{

  // Initialize the context for the object
  retval = krb5_init_context(&context);
  if (retval) {
    com_err("krb5_sock::krb5_create()", retval, "while initializing krb5");
    return 0;
  }

  (void) signal(SIGPIPE, SIG_IGN);

  // Check to see that CRC32 is supported
  if (!valid_cksumtype(CKSUMTYPE_CRC32)) {
    com_err("krb5_sock::krb5_create()", KRB5_PROG_SUMTYPE_NOSUPP, "while using CRC-32");
    return 0;
  }

  // Create the socket
  if ( (sock = socket(AF_INET, SOCK_STREAM, 0) ) == -1 ) {
    com_err("krb5_sock::krb5_create()", 0, "while creating socket");
    return 0;
  }
  
  return 1;
}

/********
Function: krb5_sock::krb5_bind
Purpose:  Calls bind and binds the socket to the port assocated with the pservice
          returns 0 on error, 1 on success.
*********/

int krb5_sock::krb5_bind(char *pservice)
{
  struct servent *serv;
  
  if (!pservice) {
    com_err("krb5_sock::krb5_bind", 0, "invalid service name");
    return 0;
  } 

  service = pservice;

  if ((serv = getservbyname(pservice, "tcp")) == (struct servent *) NULL) {
    fprintf(stderr, "service unknown: %s/tcp\n", pservice);
    return 0;
  }

  return krb5_bind(ntohs(serv->s_port), pservice);
} 

/********
Function: krb5_sock::krb5_bind
Purpose:  Calls bind and binds the socket to the port assocated with the pservice
          returns 0 on error, 1 on success.
*********/

int krb5_sock::krb5_bind(int pport, char *pservice)
{
  service = pservice;
  port = pport;

  struct sockaddr_in s_addr_in;

  // Setup the port.
  s_addr_in.sin_family = AF_INET;
  s_addr_in.sin_addr.s_addr = INADDR_ANY;
  s_addr_in.sin_port = htons(pport);

  // Bind the port to the socket
  if( bind(sock, (struct sockaddr *)&s_addr_in, sizeof(struct sockaddr_in)) == -1 )
    return 0;
 
  /* Set socket options.  We try to make the port reusable and have it
     close as fast as possible without waiting in unnecessary wait states
     on close. */
  int on = 1;
  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,(char *) &on, sizeof(on));
#ifdef SO_LINGER
  struct linger ling;
  ling.l_onoff = 1;
  ling.l_linger = 15;
  setsockopt(sock, SOL_SOCKET, SO_LINGER, (char *) &ling, sizeof(ling));
#endif /* SO_LINGER */

  return 1;
}


/*******
Function: krb5_sock::krb5_close
Purpose:  Frees the files descriptor and closes the socket connection.
*******/ 
int krb5_sock::krb5_close()
{
#ifdef DEBUG
  fprintf(stderr, "PID %d: closing file descriptor %d\n" , getpid(), sock);
#endif
  if (close(sock) == -1) 
    return 0;

  return 1;
}

/******
Function: krb5_sock::krb5_get_fd
Purpose:  Returns the file descriptor that is associated with this object.
******/

int krb5_sock::krb5_get_fd()
{
  return sock;
}

/******
Function: krb5_sock::krb5_set_fd
Purpose:  Set the file descriptor associated with this object.
*******/

void krb5_sock::krb5_set_fd(int new_fd)
{
  sock = new_fd;
}

#endif /* KERBEROS */
