/******************************************************************
krb5_sock_serv.C
HummingBird Project - Deborah A. Frincke, PhD.
Summer 1997
Coded by: Jamie Marconi
Description: This class is used for kreberized server ports.
*******************************************************************/
/*
  Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <config.h>

#ifdef KERBEROS

#include "krb5_sock_serv.h"

/********
Function: krb5_sock_serv::krb5_sock_serv
Purpose:  Basic constructor.  Init. some varibles.
********/

krb5_sock_serv::krb5_sock_serv()
{
}

/********
Function: krb5_sock_serv::~krb5_sock_serv
Purpose:  Basic destructor.
********/

krb5_sock_serv::~krb5_sock_serv()
{
  if (auth_context)
    krb5_auth_con_free(context, auth_context);

  auth_context = NULL;
  sock = 0;
  asock = 0;
}


/*******
Function: krb5_sock_serv::krb5_closea
Purpose:  Frees the files descriptor and closes the socket connection.
*******/ 
int krb5_sock_serv::krb5_closea()
{
  if (close(asock) == -1) 
    return 0;

  return 1;
}

/********
Function: krb5_sock_serv::krb5_listen
Purpose:  Calls the system function listen to make the port listen for 
connections.
********/

int krb5_sock_serv::krb5_listen(int max_connections)
{	
  if (listen(sock, max_connections) == -1) 
    return 0;
  else
    return 1;
}

/********
Function: krb5_sock_serv::krb5_accept
Purpose:  Calls the system function accpet to block until a connection to 
the port is experienced.
********/

int krb5_sock_serv::krb5_accept()
{
  if ((asock = accept(sock, (struct sockaddr *) rmt_sockaddr, &sin_len)) >= 0) {
    if (getpeername(asock, (struct sockaddr *) rmt_sockaddr, &sin_len) < 0) {
      fprintf(stderr, "getpeername error\n");
      return 1;
    }
    return 0;
 } else
    return 1;
}

/********
Function: krb5_sock_serv::krb5_m_auth
Purpose: Recieve and process Kerberos Authorization information
********/

char * krb5_sock_serv::krb5_m_auth(char *keytab_name)
{
  if (retval = krb5_kt_resolve(context, keytab_name, &keytab)) {
    com_err("krb5_sock_serv::krb5_m_auth", retval,"while resolving keytab file %s", keytab_name);
    return NULL;
  }

  krb5_principal server;
  if (retval = krb5_sname_to_principal(context, NULL, service, 
				       KRB5_NT_SRV_HST, &server)) {
    com_err("krb5_sock_serv::krb5_m_auth", retval,
	    "while generating service name (%s)", service);
    return NULL;
  }

#ifdef DEBUG
  fprintf(stderr, "rmt_sockaddr.sin_addr is %s\n", inet_ntoa(rmt_sockaddr->sin_addr));
  fprintf(stderr, "rmt_sockaddr.sin_port is %d\n", ntohs(rmt_sockaddr->sin_port)); 
#endif
  
  krb5_ticket * ticket;
  if (retval = krb5_recvauth(context, &auth_context, (krb5_pointer)&asock,
			     KRB5_SOCK_VERSION, server, 
			     0,	/* no flags */
			     keytab,	/* default keytab is NULL */
			     &ticket)) {
    com_err("krb5_sock_serv::krb5_m_auth", retval, "recvauth failed");
    return NULL;
  }

  char *cname;
  if (retval = krb5_unparse_name(context, ticket->enc_part2->client, &cname)){
    com_err("krb5_sock_serv::krb5_m_auth", retval, "unparse failed");
#ifdef DEBUG
    fprintf(stderr, "You are <unparse error>\n");
#endif
  } else {
#ifdef DEBUG
    fprintf(stderr, "You are %s\n", cname);
#endif
    //free(cname);
  }

  // Fill out a sockaddr_in for the local hostname
  //  struct sockaddr_in loc_sockaddr;
  memset((char *)loc_sockaddr, 0, sizeof(struct sockaddr_in));
  loc_sockaddr->sin_family = AF_INET;
#ifdef BROKEN_STREAMS_SOCKETS
  if (gethostname(full_loc_hname, sizeof(full_loc_hname)) < 0) {
    perror("gethostname");
    return NULL;
  }
  
  struct hostent *host;
  if ((host = gethostbyname(full_loc_hname)) == (struct hostent *)0) {
    fprintf(stderr, "%s: unknown host\n", full_loc_hname);
    return NULL;
  }
  memcpy((char *) &loc_sockaddr->sin_addr, host->h_addr, sizeof(loc_sockaddr->sin_addr));
#endif /* BROKEN_STREAMS_SOCKETS */ 
  
  int sockaddr_in_len = sizeof(struct sockaddr_in);
  if (getsockname(asock, (struct sockaddr *) loc_sockaddr, &sockaddr_in_len) < 0) {
    perror("getsockname");
    close(asock);
    return NULL;
  }
  
#ifdef DEBUG
  fprintf(stderr, "loc_sockaddr.sin_addr is %s\n", inet_ntoa(loc_sockaddr->sin_addr));
  fprintf(stderr, "loc_sockaddr.sin_port is %d\n", ntohs(loc_sockaddr->sin_port));
#endif

  krb5_address rmt_addr;
  krb5_address loc_addr;
  
  rmt_addr.addrtype = ADDRTYPE_IPPORT;
  rmt_addr.length = sizeof(rmt_sockaddr->sin_port);
  rmt_addr.contents = (krb5_octet *) &rmt_sockaddr->sin_port;

  loc_addr.addrtype = ADDRTYPE_IPPORT;
  loc_addr.length = sizeof(loc_sockaddr->sin_port);
  loc_addr.contents = (krb5_octet *) &loc_sockaddr->sin_port;

  if ((retval = krb5_auth_con_setports(context, auth_context,
				       &loc_addr, &rmt_addr))) {
    com_err("krb5_sock_serv::krb5_m_auth", retval, "while setting remote port\n");
    return NULL;
  }

  rmt_addr.addrtype = ADDRTYPE_INET;
  rmt_addr.length = sizeof(rmt_sockaddr->sin_addr);
  rmt_addr.contents = (krb5_octet *) &rmt_sockaddr->sin_addr;

  loc_addr.addrtype = ADDRTYPE_INET;
  loc_addr.length = sizeof(loc_sockaddr->sin_addr);
  loc_addr.contents = (krb5_octet *) &loc_sockaddr->sin_addr;

  if ((retval = krb5_auth_con_setaddrs(context, auth_context,
				       &loc_addr, &rmt_addr))) {
    com_err("krb5_sock_serv::krb5_m_auth", retval, "while setting remote addr\n");
    return NULL;
  }

  return cname;
}


/********
Function: krb5_sock_serv::krb5_get_asock
Purpose:  Returns the new file descriptor of the accepted socket.
********/

int krb5_sock_serv::krb5_get_asock(void)
{
  return asock;
}

/********
Function: krb5_sock_serv::krb5_get_rmt_addr
Purpose:  Gets the IP of the connecting host.
********/

void krb5_sock_serv::krb5_get_rmt_addr(char *buf)
{
  strcpy(buf,inet_ntoa(rmt_sockaddr->sin_addr));
} 


/********
Function: krb5_sock_serv::krb5_mk_cli
Purpose:  Create a krb5_sock_cli object from the accepted socket
          Auth info is not copied to client object
*********/
krb5_sock_cli *krb5_sock_serv::krb5_mk_cli()
{
  krb5_sock_cli *nc;
  nc = new krb5_sock_cli(asock, service);
  
  return nc;
}
  
/**********
Function: krb5_sock_cli::krb5_e_read
Purpose:  Reads the data from the accepted socket unencryped and source verified then,
          puts it in the buffer.
          Returns 0 on error, 1 otherwise
**********/
int krb5_sock_serv::krb5_e_read(char *buf, size_t buflen)
{
  // unsigned char pktbuf[BUFSIZ];
  krb5_data packet;
  krb5_data message;
  
  int i = sizeof(struct sockaddr_in);
  //  struct sockaddr_in rmt_sockaddr;
  if ((i = recvfrom(asock, (char *)buf, buflen-1, 0,
		    (struct sockaddr *)rmt_sockaddr, &i)) < 0) {
    perror("receiving datagram");
    return 0;
  }

#ifdef DEBUG
  fprintf(stderr, "Received %d bytes\n", i);
#endif
  
  packet.length = i;
  packet.data = buf;//(krb5_pointer) buf;
  
  if ((retval = krb5_rd_priv(context, auth_context, &packet,
			     &message, NULL))) {
    com_err("krb5_sock_serv::krb5_e_read", retval, "while verifying PRIV message");
    return 0;
  }
  
  strcpy(buf, message.data);
  return message.length;
}


/**********
Function: krb5_sock_cli::krb5_e_write
Purpose:  Writes message to the accepted socket encryped and source verified then,
          Returns 1 on error, 0 otherwise
**********/
int krb5_sock_serv::krb5_e_write(char *buf)
{
  krb5_data packet;
  krb5_data inbuf;

  inbuf.data = buf;
  inbuf.length = strlen(buf);

  if ((retval = krb5_mk_priv(context, auth_context, &inbuf,
			     &packet, NULL))) {
    com_err("krb5_sock_serv::krb5_e_write", retval, "while making KRB_PRIV message");
    return 0;
  }

  int i;
  if ((i = send(asock, (char *)packet.data, packet.length, 0)) < 0)
	com_err("krb5_sock_serv::krb5_e_write", errno, "while sending PRIV message");

  krb5_xfree(packet.data);
  
  return i;
}

#endif /* KERBEROS */
